@@ -83,6 +83,14 @@ class AdminForm:
     def non_field_errors(self):
         return self.form.non_field_errors
 
+    @property
+    def fields(self):
+        return self.form.fields
+
+    @property
+    def is_bound(self):
+        return self.form.is_bound
+
     @property
     def media(self):
         media = self.form.media
@@ -433,9 +441,16 @@ class InlineAdminFormSet:
     def forms(self):
         return self.formset.forms
 
-    @property
     def non_form_errors(self):
-        return self.formset.non_form_errors
+        return self.formset.non_form_errors()
+
+    @property
+    def is_bound(self):
+        return self.formset.is_bound
+
+    @property
+    def total_form_count(self):
+        return self.formset.total_form_count
 
     @property
     def media(self):
