@@ -78,7 +78,7 @@ def response_for_exception(request, exc):
             request.path,
             response=response,
             request=request,
-            exc_info=sys.exc_info(),
+            exception=exc,
         )
 
     elif isinstance(exc, MultiPartParserError):
@@ -90,7 +90,7 @@ def response_for_exception(request, exc):
             request.path,
             response=response,
             request=request,
-            exc_info=sys.exc_info(),
+            exception=exc,
         )
 
     elif isinstance(exc, BadRequest):
@@ -108,7 +108,7 @@ def response_for_exception(request, exc):
             request.path,
             response=response,
             request=request,
-            exc_info=sys.exc_info(),
+            exception=exc,
         )
     elif isinstance(exc, SuspiciousOperation):
         if isinstance(exc, (RequestDataTooBig, TooManyFieldsSent)):
@@ -123,6 +123,7 @@ def response_for_exception(request, exc):
         )
         security_logger.error(
             str(exc),
+            exc_info=exc,
             extra={"status_code": 400, "request": request},
         )
         if settings.DEBUG:
@@ -145,7 +146,7 @@ def response_for_exception(request, exc):
             request.path,
             response=response,
             request=request,
-            exc_info=sys.exc_info(),
+            exception=exc,
         )
 
     # Force a TemplateResponse to be rendered.
