@@ -8,7 +8,7 @@
 import datetime
 
 from django.utils.regex_helper import _lazy_re_compile
-from django.utils.timezone import get_fixed_timezone, utc
+from django.utils.timezone import get_fixed_timezone
 
 date_re = _lazy_re_compile(r"(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2})$")
 
@@ -40,11 +40,11 @@ standard_duration_re = _lazy_re_compile(
 iso8601_duration_re = _lazy_re_compile(
     r"^(?P<sign>[-+]?)"
     r"P"
-    r"(?:(?P<days>\d+(.\d+)?)D)?"
+    r"(?:(?P<days>\d+([\.,]\d+)?)D)?"
     r"(?:T"
-    r"(?:(?P<hours>\d+(.\d+)?)H)?"
-    r"(?:(?P<minutes>\d+(.\d+)?)M)?"
-    r"(?:(?P<seconds>\d+(.\d+)?)S)?"
+    r"(?:(?P<hours>\d+([\.,]\d+)?)H)?"
+    r"(?:(?P<minutes>\d+([\.,]\d+)?)M)?"
+    r"(?:(?P<seconds>\d+([\.,]\d+)?)S)?"
     r")?"
     r"$"
 )
@@ -118,7 +118,7 @@ def parse_datetime(value):
             kw["microsecond"] = kw["microsecond"] and kw["microsecond"].ljust(6, "0")
             tzinfo = kw.pop("tzinfo")
             if tzinfo == "Z":
-                tzinfo = utc
+                tzinfo = datetime.timezone.utc
             elif tzinfo is not None:
                 offset_mins = int(tzinfo[-2:]) if len(tzinfo) > 3 else 0
                 offset = 60 * int(tzinfo[1:3]) + offset_mins
@@ -147,12 +147,6 @@ def parse_duration(value):
         sign = -1 if kw.pop("sign", "+") == "-" else 1
         if kw.get("microseconds"):
             kw["microseconds"] = kw["microseconds"].ljust(6, "0")
-        if (
-            kw.get("seconds")
-            and kw.get("microseconds")
-            and kw["seconds"].startswith("-")
-        ):
-            kw["microseconds"] = "-" + kw["microseconds"]
         kw = {k: float(v.replace(",", ".")) for k, v in kw.items() if v is not None}
         days = datetime.timedelta(kw.pop("days", 0.0) or 0.0)
         if match.re == iso8601_duration_re:
